import adts.*;
import java.util.*;

/**
 * Clase con algoritmos en grafos
 * 
 */
public class OperacionesEnGrafos
{

    /**
     * Busqueda en profundidad desde un nudo origen
     */
    public static <V,A> void buscaEnProfundidad 
        (Grafo<V,A> g, int[] antecesor, boolean[] visitado, int origen)
        throws IdIncorrecto
    {
        visitado[origen]=true;
        List<Arista<A>> ady=g.listaAristas(origen);
        for (Arista<A> a:ady) {
            int v=a.destino();
            if (!visitado[v]) {
                antecesor[v]=origen;
                buscaEnProfundidad(g, antecesor, visitado, v);
            }
        }
        System.out.print(g.contenido(origen)+" ");
    }
    
    /** 
     * Muestra todos los vertices del grafo siguiendo una busqueda 
     * en profundidad, con origenes ordenados por idVertice
     */
    public static <V,A> void muestraEnProfundidad (Grafo<V,A> g)
    {
        try {
            boolean[] visitado=new boolean[g.numVertices()];
            // en java visitado se inicializa a false
            int[] antecesor=new int[g.numVertices()];
            for (int i=0; i<g.numVertices(); i++) {
                if (!visitado[i]) {
                    antecesor[i]=-1; // para indicar que no tiene
                    buscaEnProfundidad(g, antecesor, visitado, i);
                    System.out.println();
                }
            }
        } catch (IdIncorrecto e) {
            System.out.println("Error inesperado "+e);
        }
    }

    /** 
     * Muestra los vertices del grafo alcanzables desde el origen indicado,
     * siguiendo una busqueda en profundidad
     */
    public static <V,A> void muestraEnProfundidad (Grafo<V,A> g, V origen)
        throws NoExiste
    {
        try {
            boolean[] visitado=new boolean[g.numVertices()];
            // en java visitado se inicializa a false
            int[] antecesor=new int[g.numVertices()];
            int i=g.idVertice(origen);
            antecesor[i]=-1; // para indicar que no tiene
            buscaEnProfundidad(g, antecesor, visitado, i);
            System.out.println();
        } catch (IdIncorrecto e) {
            System.out.println("Error inesperado "+e);
        }
    }    
    /**
     * Obtener el grafo Traspuesto
     */
    public static <V,A> Grafo<V,A> traspuesto(Grafo<V,A> g) {
        Grafo<V,A> nuevo = new GrafoAdyacencia<V,A>();
        try {
            for(int i=0; i<g.numVertices(); i++) {
                List<Arista<A>> ady=g.listaAristas(i);   
                for(Arista<A> a:ady) {
                    nuevo.nuevaArista(g.contenido(a.destino()),
                       g.contenido(a.origen()),a.contenido());   
                }
            }
        } catch (IdIncorrecto e) {
            System.out.println("Error inesperado "+e);
        }
        return nuevo;
    }
    
    /**
     * Mostrar el grafo en pantalla
     */
    public static <V,A> void muestra(Grafo<V,A> g) {
        try {
            System.out.println();
            System.out.println("------------Grafo---------");
            for (int i=0; i<g.numVertices(); i++) {
                System.out.print(g.contenido(i)+" - ");
                List<Arista<A>> lista=g.listaAristas(i);
                for (Arista<A> a:lista) {
                    System.out.print(g.contenido(a.destino())+"("+a.contenido()+") ");
                }
                System.out.println();
            }
        } catch (IdIncorrecto e) {
            System.out.println("Error inesperado: "+e);
        }
    }
    
}
